@@ -1,23 +1,22 @@
+# -*- coding: utf-8 -*-
 """Google class file."""
 
-import argparse
+from __future__ import print_function
+
 import hashlib
-import json
+import os
+import pickle
 import re
-import requests
 import string
 import sys
-import time
-from httplib2 import Http
-from random import choice
 
 import google.auth
+from google_auth_oauthlib.flow import InstalledAppFlow
+from google.auth.transport.requests import Request
 from google.oauth2 import service_account
 from googleapiclient.errors import HttpError
+from random import choice
 
-# used for auth_stored_credentials
-from oauth2client import client, tools
-from oauth2client.file import Storage as FileStorage
 from oauth2client.service_account import ServiceAccountCredentials
 
 from bits.helpers import chunks
@@ -25,37 +24,6 @@ from bits.helpers import chunks
 # bits-google model subclasses
 from .models import Models
 
-# bits-google service subclasses
-from .services.bigquery import BigQuery
-from .services.billing import CloudBilling
-from .services.budgets import CloudBillingBudgets
-from .services.cai import CloudAssetInventory
-from .services.calendar import Calendar
-from .services.compute import Compute
-from .services.crm import CloudResourceManager
-from .services.datastore import Datastore
-from .services.directory import Directory
-from .services.dns import CloudDNS
-from .services.drive import Drive
-from .services.firestore import Firestore
-from .services.gmail import Gmail
-from .services.groupssettings import GroupsSettings
-from .services.iam import IAM
-from .services.iamcredentials import IAMCredentials
-from .services.labelmanager import LabelManager
-from .services.licensing import Licensing
-from .services.people import People
-from .services.privatecatalog import PrivateCatalog
-from .services.recommender import Recommender
-from .services.reports import Reports
-from .services.resourcesearch import ResourceSearch
-from .services.securitycenter import SecurityCenter
-from .services.serviceusage import ServiceUsage
-from .services.sheets import Sheets
-from .services.sql import SQL
-from .services.storage import Storage
-from .services.vault import Vault
-
 
 class Google(object):
     """Google class definition."""
@@ -67,7 +35,7 @@ class Google(object):
         app_name=None,
         client_scopes=None,
         client_secrets_file='client_secrets.json',
-        credentials_file='credentials.json',
+        credentials_file='credentials.pickle',
         gsuite_licenses={},
         private_key_password='notasecret',
         redirect_uri='urn:ietf:wg:oauth:2.0:oob',
@@ -192,33 +160,28 @@ class Google(object):
 
     def auth_stored_credentials(self, scopes=[]):
         """Authorize stored credentials."""
-        # this has a dependency on oauth2client
+        credentials = None
         if not scopes:
             scopes = self.client_scopes
-        try:
-            parser = argparse.ArgumentParser(parents=[tools.argparser])
-            parser.add_argument('args', nargs=argparse.REMAINDER)
-            flags = parser.parse_args()
-            flags.noauth_local_webserver = True
-        except ImportError:
-            flags = None
-        # retrieve stored credentials, if they exist
-        store = FileStorage(self.credentials_file)
-        credentials = store.get()
-        # create stored credentials
-        if not credentials or credentials.invalid:
-            flow = client.flow_from_clientsecrets(
-                self.client_secrets_file,
-                scopes,
-            )
-            flow.user_agent = self.app_name
-            if flags:
-                credentials = tools.run_flow(flow, store, flags)
-            if self.verbose:
-                print('Saved credentials to %s' % (self.credentials_file))
-        # authorize an Http instance
-        self.http = credentials.authorize(Http())
-        # store credentials
+        # The file token.pickle stores the user's access and refresh tokens, and is
+        # created automatically when the authorization flow completes for the first
+        # time.
+        if os.path.exists(self.credentials_file):
+            with open(self.credentials_file, 'rb') as token:
+                credentials = pickle.load(token)
+        # If there are no (valid) credentials available, let the user log in.
+        if not credentials or not credentials.valid:
+            if credentials and credentials.expired and credentials.refresh_token:
+                credentials.refresh(Request())
+            else:
+                flow = InstalledAppFlow.from_client_secrets_file(
+                    self.client_secrets_file,
+                    scopes
+                )
+                credentials = flow.run_console()
+            # Save the credentials for the next run
+            with open(self.credentials_file, 'wb') as token:
+                pickle.dump(credentials, token)
         self.credentials = credentials
         return credentials
 
@@ -240,118 +203,182 @@ class Google(object):
     #
     def bigquery(self):
         """Return the BigQuery class."""
+        from .services.bigquery import BigQuery
         return BigQuery(self.credentials)
 
     def billing(self):
         """Return the CloudBilling class."""
+        from .services.billing import CloudBilling
         return CloudBilling(self.credentials, self.api_key)
 
     def budgets(self):
         """Return the CloudBillingBudgets class."""
+        from .services.budgets import CloudBillingBudgets
         return CloudBillingBudgets(self.credentials)
 
     def cai(self):
         """Return the CloudAssetInventory class."""
+        from .services.cai import CloudAssetInventory
         return CloudAssetInventory(self.credentials)
 
     def calendar(self):
         """Return the Calendar class."""
+        from .services.calendar import Calendar
         return Calendar(self.credentials)
 
+    def cloudbuild(self):
+        """Return the CloudBuild class."""
+        from .services.cloudbuild import CloudBuild
+        return CloudBuild(self.credentials)
+
+    def cloudprint(self):
+        """Return the CloudPrint class."""
+        from .services.cloudprint import CloudPrint
+        return CloudPrint(self.credentials)
+
+    def cloudsearch(self):
+        """Return the CloudSearch class."""
+        from .services.cloudsearch import CloudSearch
+        return CloudSearch(self.credentials)
+
     def compute(self):
         """Return the Compute class."""
+        from .services.compute import Compute
         return Compute(self.credentials)
 
     def crm(self):
         """Return the CloudResourceManager class."""
+        from .services.crm import CloudResourceManager
         return CloudResourceManager(self.credentials)
 
     def datastore(self, project):
         """Return the Datastore class."""
+        from .services.datastore import Datastore
         return Datastore(project, self.credentials)
 
     def directory(self):
         """Return the Directory class."""
+        from .services.directory import Directory
         return Directory(self.credentials)
 
     def dns(self):
         """Return the CloudDNS class."""
+        from .services.dns import CloudDNS
         return CloudDNS(self.credentials)
 
     def drive(self):
         """Return the Drive class."""
+        from .services.drive import Drive
         return Drive(self.credentials)
 
     def firestore(self, project):
         """Return the Firestore class."""
+        from .services.firestore import Firestore
         return Firestore(project, self.credentials)
 
     def gmail(self):
         """Return the Gmail class."""
+        from .services.gmail import Gmail
         return Gmail(self.credentials)
 
     def groupssettings(self):
         """Return the GroupsSettings class."""
+        from .services.groupssettings import GroupsSettings
         return GroupsSettings(self.credentials)
 
     def iam(self):
         """Return the IAM class."""
+        from .services.iam import IAM
         return IAM(self.credentials)
 
     def iamcredentials(self):
         """Return the IAMCredentials class."""
+        from .services.iamcredentials import IAMCredentials
         return IAMCredentials(self.credentials)
 
+    def identity(self):
+        """Return the CloudIdentity class."""
+        from .services.identity import CloudIdentity
+        return CloudIdentity(self.credentials)
+
     def labelmanager(self):
         """Return the LabelManager class."""
+        from .services.labelmanager import LabelManager
         return LabelManager(self.credentials, self.api_key)
 
     def licensing(self):
         """Return the Licensing class."""
-        return Licensing(self.http, self.gsuite_licenses)
+        from .services.licensing import Licensing
+        return Licensing(self.credentials, self.gsuite_licenses)
 
     def people(self):
         """Return the People class."""
+        from .services.people import People
         return People(self.credentials)
 
     def privatecatalog(self):
         """Return the PrivateCatalog class."""
+        from .services.privatecatalog import PrivateCatalog
         return PrivateCatalog(self.credentials, self.api_key)
 
+    def pubsub(self):
+        """Return the PubSub class."""
+        from .services.pubsub import PubSub
+        return PubSub(self.credentials)
+
     def recommender(self):
         """Return the Recommender class."""
+        from .services.recommender import Recommender
         return Recommender(self.credentials)
 
     def reports(self):
         """Return the Reports class."""
+        from .services.reports import Reports
         return Reports(self.credentials)
 
     def resourcesearch(self):
         """Return the ResourceSearch class."""
+        from .services.resourcesearch import ResourceSearch
         return ResourceSearch(self.credentials)
 
-    def sheets(self):
-        """Return the Sheets class."""
-        return Sheets(self.credentials)
+    def secretmanager(self):
+        """Return the SecretManager class."""
+        from .services.secretmanager import SecretManager
+        return SecretManager(self.credentials)
 
     def securitycenter(self):
         """Return the SecurityCenter class."""
+        from .services.securitycenter import SecurityCenter
         return SecurityCenter(self.credentials)
 
     def serviceusage(self):
         """Return the ServiceUsage class."""
+        from .services.serviceusage import ServiceUsage
         return ServiceUsage(self.credentials)
 
+    def sheets(self):
+        """Return the Sheets class."""
+        from .services.sheets import Sheets
+        return Sheets(self.credentials)
+
+    def sourcerepo(self):
+        """Return the SourceRepo class."""
+        from .services.sourcerepo import SourceRepo
+        return SourceRepo(self.credentials)
+
     def sql(self):
         """Return the SQL class."""
+        from .services.sql import SQL
         return SQL(self.credentials)
 
     def storage(self):
         """Return the Storage class."""
+        from .services.storage import Storage
         return Storage(self.credentials)
 
     def vault(self):
         """Return the Vault class."""
+        from .services.vault import Vault
         return Vault(self.credentials)
 
     #
@@ -900,7 +927,7 @@ class Google(object):
                 success.append(groupKey)
                 print('      - Unsubscribed from group: %s' % (groupKey))
             except Exception as e:
-                print >> sys.stderr, e
+                print(e, file=sys.stderr)
 
         return success
 
@@ -926,7 +953,7 @@ class Google(object):
                 success.append(groupKey)
                 print('      + Subscribed to group: %s' % (groupKey))
             except Exception as e:
-                print >> sys.stderr, e
+                print(e, file=sys.stderr)
 
         return success
 
@@ -1110,7 +1137,7 @@ class Google(object):
             password = ''.join(choice(characters) for _ in range(length))
 
         # generate a md5 of the password
-        md5 = hashlib.md5(password).hexdigest()
+        md5 = hashlib.md5(password.encode('utf-8')).hexdigest()
 
         # update the user with the new password
         return self.directory().change_password(
@@ -1225,6 +1252,10 @@ class Google(object):
             }
             return bitsdb
 
+        def sharedcontact(self, g):
+            """Return a google#sharedcontact for BITSdb API."""
+            return g
+
         def user(self, g):
             """Return a google#user for BITSdb API."""
             bitsdb = {
@@ -1245,6 +1276,8 @@ class Google(object):
                     bitsdb[k] = self.group(s)
                 elif collection == 'billingaccounts':
                     bitsdb[k] = self.billingaccount(s)
+                elif collection == 'sharedcontacts':
+                    bitsdb[k] = self.sharedcontact(s)
                 elif collection == 'users':
                     bitsdb[k] = self.user(s)
                 else:
