@@ -16,10 +16,11 @@ class PrivateCatalog(Base):
             credentials=credentials,
             discoveryServiceUrl='https://cloudprivatecatalog.googleapis.com/$discovery/rest?version=v1alpha1&key=%s' % (
                 api_key
-            )
+            ),
+            cache_discovery=False
         )
-        self.catalogproducer = build('cloudprivatecatalogproducer', 'v1beta1', credentials=credentials)
-        self.catalog = build('cloudprivatecatalog', 'v1beta1', credentials=credentials)
+        self.catalogproducer = build('cloudprivatecatalogproducer', 'v1beta1', credentials=credentials, cache_discovery=False)
+        self.catalog = build('cloudprivatecatalog', 'v1beta1', credentials=credentials, cache_discovery=False)
 
     def get_catalogs(self, resource):
         """Return a list of catalogs."""
